@@ -1,20 +1,21 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*  GMime
- *  Copyright (C) 1999-2006 Jeffrey Stedfast
+ *  Copyright (C) 2000-2009 Jeffrey Stedfast
  *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
- *  (at your option) any later version.
+ *  This library is free software; you can redistribute it and/or
+ *  modify it under the terms of the GNU Lesser General Public License
+ *  as published by the Free Software Foundation; either version 2.1
+ *  of the License, or (at your option) any later version.
  *
- *  This program is distributed in the hope that it will be useful,
+ *  This library is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ *  Lesser General Public License for more details.
  *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Street #330, Boston, MA 02111-1307, USA.
+ *  You should have received a copy of the GNU Lesser General Public
+ *  License along with this library; if not, write to the Free
+ *  Software Foundation, 51 Franklin Street, Fifth Floor, Boston, MA
+ *  02110-1301, USA.
  */
 
 
@@ -26,7 +27,19 @@
 
 #include "gmime-message-part.h"
 
-#define d(x) x
+#define d(x)
+
+
+/**
+ * SECTION: gmime-message-part
+ * @title: GMimeMessagePart
+ * @short_description: Message parts
+ * @see_also:
+ *
+ * A #GMimeMessagePart represents message/rfc822 and message/news MIME
+ * parts.
+ **/
+
 
 /* GObject class methods */
 static void g_mime_message_part_class_init (GMimeMessagePartClass *klass);
@@ -34,11 +47,11 @@ static void g_mime_message_part_init (GMimeMessagePart *message_part, GMimeMessa
 static void g_mime_message_part_finalize (GObject *object);
 
 /* GMimeObject class methods */
-static void message_part_init (GMimeObject *object);
-static void message_part_add_header (GMimeObject *object, const char *header, const char *value);
+static void message_part_prepend_header (GMimeObject *object, const char *header, const char *value);
+static void message_part_append_header (GMimeObject *object, const char *header, const char *value);
 static void message_part_set_header (GMimeObject *object, const char *header, const char *value);
 static const char *message_part_get_header (GMimeObject *object, const char *header);
-static void message_part_remove_header (GMimeObject *object, const char *header);
+static gboolean message_part_remove_header (GMimeObject *object, const char *header);
 static void message_part_set_content_type (GMimeObject *object, GMimeContentType *content_type);
 static char *message_part_get_headers (GMimeObject *object);
 static ssize_t message_part_write_to_stream (GMimeObject *object, GMimeStream *stream);
@@ -61,7 +74,7 @@ g_mime_message_part_get_type (void)
 			NULL, /* class_finalize */
 			NULL, /* class_data */
 			sizeof (GMimeMessagePart),
-			16,   /* n_preallocs */
+			0,    /* n_preallocs */
 			(GInstanceInitFunc) g_mime_message_part_init,
 		};
 		
@@ -82,11 +95,11 @@ g_mime_message_part_class_init (GMimeMessagePartClass *klass)
 	
 	gobject_class->finalize = g_mime_message_part_finalize;
 	
-	object_class->init = message_part_init;
-	object_class->add_header = message_part_add_header;
+	object_class->prepend_header = message_part_prepend_header;
+	object_class->append_header = message_part_append_header;
+	object_class->remove_header = message_part_remove_header;
 	object_class->set_header = message_part_set_header;
 	object_class->get_header = message_part_get_header;
-	object_class->remove_header = message_part_remove_header;
 	object_class->set_content_type = message_part_set_content_type;
 	object_class->get_headers = message_part_get_headers;
 	object_class->write_to_stream = message_part_write_to_stream;
@@ -109,22 +122,22 @@ g_mime_message_part_finalize (GObject *object)
 	G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
-
 static void
-message_part_init (GMimeObject *object)
-{
-	/* no-op */
-	GMIME_OBJECT_CLASS (parent_class)->init (object);
-}
-
-static void
-message_part_add_header (GMimeObject *object, const char *header, const char *value)
+message_part_prepend_header (GMimeObject *object, const char *header, const char *value)
 {
 	/* Make sure that the header is a Content-* header, else it
            doesn't belong on a message part */
-	
 	if (!g_ascii_strncasecmp ("Content-", header, 8))
-		GMIME_OBJECT_CLASS (parent_class)->add_header (object, header, value);
+		GMIME_OBJECT_CLASS (parent_class)->prepend_header (object, header, value);
+}
+
+static void
+message_part_append_header (GMimeObject *object, const char *header, const char *value)
+{
+	/* Make sure that the header is a Content-* header, else it
+           doesn't belong on a message part */
+	if (!g_ascii_strncasecmp ("Content-", header, 8))
+		GMIME_OBJECT_CLASS (parent_class)->append_header (object, header, value);
 }
 
 static void
@@ -136,7 +149,6 @@ message_part_set_header (GMimeObject *object, const char *header, const char *va
 	
 	/* Make sure that the header is a Content-* header, else it
            doesn't belong on a message part */
-	
 	if (!g_ascii_strncasecmp ("Content-", header, 8))
 		GMIME_OBJECT_CLASS (parent_class)->set_header (object, header, value);
 }
@@ -146,21 +158,21 @@ message_part_get_header (GMimeObject *object, const char *header)
 {
 	/* Make sure that the header is a Content-* header, else it
            doesn't belong on a message part */
-	
 	if (!g_ascii_strncasecmp ("Content-", header, 8))
 		return GMIME_OBJECT_CLASS (parent_class)->get_header (object, header);
 	else
 		return NULL;
 }
 
-static void
+static gboolean
 message_part_remove_header (GMimeObject *object, const char *header)
 {
 	/* Make sure that the header is a Content-* header, else it
            doesn't belong on a message part */
+	if (g_ascii_strncasecmp ("Content-", header, 8) != 0)
+		return FALSE;
 	
-	if (!g_ascii_strncasecmp ("Content-", header, 8))
-		GMIME_OBJECT_CLASS (parent_class)->remove_header (object, header);
+	return GMIME_OBJECT_CLASS (parent_class)->remove_header (object, header);
 }
 
 static void
@@ -183,7 +195,7 @@ message_part_write_to_stream (GMimeObject *object, GMimeStream *stream)
 	ssize_t nwritten, total = 0;
 	
 	/* write the content headers */
-	if ((nwritten = g_mime_header_write_to_stream (object->headers, stream)) == -1)
+	if ((nwritten = g_mime_header_list_write_to_stream (object->headers, stream)) == -1)
 		return -1;
 	
 	total += nwritten;
@@ -213,19 +225,20 @@ message_part_write_to_stream (GMimeObject *object, GMimeStream *stream)
  * Creates a new MIME message part object with a default content-type
  * of message/@subtype.
  *
- * Returns an empty MIME message part object with a default
+ * Returns: an empty MIME message part object with a default
  * content-type of message/@subtype.
  **/
 GMimeMessagePart *
 g_mime_message_part_new (const char *subtype)
 {
+	GMimeContentType *content_type;
 	GMimeMessagePart *part;
-	GMimeContentType *type;
 	
-	part = g_object_new (GMIME_TYPE_MESSAGE_PART, NULL, NULL);
+	part = g_object_newv (GMIME_TYPE_MESSAGE_PART, 0, NULL);
 	
-	type = g_mime_content_type_new ("message", subtype ? subtype : "rfc822");
-	g_mime_object_set_content_type (GMIME_OBJECT (part), type);
+	content_type = g_mime_content_type_new ("message", subtype ? subtype : "rfc822");
+	g_mime_object_set_content_type (GMIME_OBJECT (part), content_type);
+	g_object_unref (content_type);
 	
 	return part;
 }
@@ -239,7 +252,7 @@ g_mime_message_part_new (const char *subtype)
  * Creates a new MIME message part object with a default content-type
  * of message/@subtype containing @message.
  *
- * Returns a MIME message part object with a default content-type of
+ * Returns: a MIME message part object with a default content-type of
  * message/@subtype containing @message.
  **/
 GMimeMessagePart *
@@ -283,14 +296,12 @@ g_mime_message_part_set_message (GMimeMessagePart *part, GMimeMessage *message)
  *
  * Gets the message object on the message part object @part.
  *
- * Returns the message part contained within @part.
+ * Returns: the message part contained within @part.
  **/
 GMimeMessage *
 g_mime_message_part_get_message (GMimeMessagePart *part)
 {
 	g_return_val_if_fail (GMIME_IS_MESSAGE_PART (part), NULL);
 	
-	g_object_ref (part->message);
-	
 	return part->message;
 }
