@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*  GMime
- *  Copyright (C) 2000-2008 Jeffrey Stedfast
+ *  Copyright (C) 2000-2009 Jeffrey Stedfast
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Lesser General Public License
@@ -23,6 +23,15 @@
 #include <config.h>
 #endif
 
+#include <glib.h>
+
+#ifdef G_OS_WIN32
+#include <fcntl.h>
+#include <io.h>
+#endif
+
+#include <errno.h>
+
 #include "gmime-stream-file.h"
 
 
@@ -131,8 +140,15 @@ stream_read (GMimeStream *stream, char *buf, size_t len)
 	GMimeStreamFile *fstream = (GMimeStreamFile *) stream;
 	size_t nread;
 	
-	if (stream->bound_end != -1 && stream->position >= stream->bound_end)
+	if (fstream->fp == NULL) {
+		errno = EBADF;
 		return -1;
+	}
+	
+	if (stream->bound_end != -1 && stream->position >= stream->bound_end) {
+		errno = EINVAL;
+		return -1;
+	}
 	
 	if (stream->bound_end != -1)
 		len = MIN (stream->bound_end - stream->position, (gint64) len);
@@ -152,15 +168,22 @@ stream_write (GMimeStream *stream, const char *buf, size_t len)
 	GMimeStreamFile *fstream = (GMimeStreamFile *) stream;
 	size_t nwritten;
 	
-	if (stream->bound_end != -1 && stream->position >= stream->bound_end)
+	if (fstream->fp == NULL) {
+		errno = EBADF;
 		return -1;
+	}
+	
+	if (stream->bound_end != -1 && stream->position >= stream->bound_end) {
+		errno = EINVAL;
+		return -1;
+	}
 	
 	if (stream->bound_end != -1)
 		len = MIN (stream->bound_end - stream->position, (gint64) len);
 	
 	/* make sure we are at the right position */
 	fseek (fstream->fp, (long) stream->position, SEEK_SET);
-	
+
 	if ((nwritten = fwrite (buf, 1, len, fstream->fp)) > 0)
 		stream->position += nwritten;
 	
@@ -172,7 +195,10 @@ stream_flush (GMimeStream *stream)
 {
 	GMimeStreamFile *fstream = (GMimeStreamFile *) stream;
 	
-	g_return_val_if_fail (fstream->fp != NULL, -1);
+	if (fstream->fp == NULL) {
+		errno = EBADF;
+		return -1;
+	}
 	
 	return fflush (fstream->fp);
 }
@@ -197,7 +223,8 @@ stream_eos (GMimeStream *stream)
 {
 	GMimeStreamFile *fstream = (GMimeStreamFile *) stream;
 	
-	g_return_val_if_fail (fstream->fp != NULL, TRUE);
+	if (fstream->fp == NULL)
+		return TRUE;
 	
 	if (stream->bound_end == -1)
 		return feof (fstream->fp) ? TRUE : FALSE;
@@ -210,8 +237,10 @@ stream_reset (GMimeStream *stream)
 {
 	GMimeStreamFile *fstream = (GMimeStreamFile *) stream;
 	
-	if (fstream->fp == NULL)
+	if (fstream->fp == NULL) {
+		errno = EBADF;
 		return -1;
+	}
 	
 	if (stream->position == stream->bound_start)
 		return 0;
@@ -229,7 +258,10 @@ stream_seek (GMimeStream *stream, gint64 offset, GMimeSeekWhence whence)
 	gint64 real = stream->position;
 	FILE *fp = fstream->fp;
 	
-	g_return_val_if_fail (fstream->fp != NULL, -1);
+	if (fstream->fp == NULL) {
+		errno = EBADF;
+		return -1;
+	}
 	
 	switch (whence) {
 	case GMIME_STREAM_SEEK_SET:
@@ -258,11 +290,15 @@ stream_seek (GMimeStream *stream, gint64 offset, GMimeSeekWhence whence)
 	}
 	
 	/* sanity check the resultant offset */
-	if (real < stream->bound_start)
+	if (real < stream->bound_start) {
+		errno = EINVAL;
 		return -1;
+	}
 	
-	if (stream->bound_end != -1 && real > stream->bound_end)
+	if (stream->bound_end != -1 && real > stream->bound_end) {
+		errno = EINVAL;
 		return -1;
+	}
 	
 	if (fseek (fp, (long) real, SEEK_SET) == -1 || (real = ftell (fp)) == -1)
 		return -1;
@@ -284,6 +320,11 @@ stream_length (GMimeStream *stream)
 	GMimeStreamFile *fstream = (GMimeStreamFile *) stream;
 	gint64 bound_end;
 	
+	if (fstream->fp == NULL) {
+		errno = EBADF;
+		return -1;
+	}
+	
 	if (stream->bound_start != -1 && stream->bound_end != -1)
 		return stream->bound_end - stream->bound_start;
 	
@@ -291,8 +332,10 @@ stream_length (GMimeStream *stream)
 	bound_end = ftell (fstream->fp);
 	fseek (fstream->fp, (long) stream->position, SEEK_SET);
 	
-	if (bound_end < stream->bound_start)
+	if (bound_end < stream->bound_start) {
+		errno = EINVAL;
 		return -1;
+	}
 	
 	return bound_end - stream->bound_start;
 }
@@ -313,11 +356,11 @@ stream_substream (GMimeStream *stream, gint64 start, gint64 end)
 
 /**
  * g_mime_stream_file_new:
- * @fp: file pointer
+ * @fp: a FILE pointer
  *
- * Creates a new GMimeStreamFile object around @fp.
+ * Creates a new #GMimeStreamFile object around @fp.
  *
- * Note: The created GMimeStreamFile object will own the FILE pointer
+ * Note: The created #GMimeStreamFile object will own the FILE pointer
  * passed in.
  *
  * Returns: a stream using @fp.
@@ -328,6 +371,10 @@ g_mime_stream_file_new (FILE *fp)
 	GMimeStreamFile *fstream;
 	gint64 start;
 	
+#ifdef G_OS_WIN32
+	_setmode (_fileno (fp), O_BINARY);
+#endif
+	
 	if ((start = ftell (fp)) == -1)
 		start = 0;
 	
@@ -342,14 +389,14 @@ g_mime_stream_file_new (FILE *fp)
 
 /**
  * g_mime_stream_file_new_with_bounds:
- * @fp: file pointer
+ * @fp: a FILE pointer
  * @start: start boundary
  * @end: end boundary
  *
- * Creates a new GMimeStreamFile object around @fp with bounds @start
+ * Creates a new #GMimeStreamFile object around @fp with bounds @start
  * and @end.
  *
- * Note: The created GMimeStreamFile object will own the FILE pointer
+ * Note: The created #GMimeStreamFile object will own the FILE pointer
  * passed in.
  *
  * Returns: a stream using @fp with bounds @start and @end.
@@ -359,6 +406,10 @@ g_mime_stream_file_new_with_bounds (FILE *fp, gint64 start, gint64 end)
 {
 	GMimeStreamFile *fstream;
 	
+#ifdef G_OS_WIN32
+	_setmode (_fileno (fp), O_BINARY);
+#endif
+	
 	fstream = g_object_newv (GMIME_TYPE_STREAM_FILE, 0, NULL);
 	g_mime_stream_construct (GMIME_STREAM (fstream), start, end);
 	fstream->owner = TRUE;
@@ -370,7 +421,7 @@ g_mime_stream_file_new_with_bounds (FILE *fp, gint64 start, gint64 end)
 
 /**
  * g_mime_stream_file_get_owner:
- * @stream: file stream
+ * @stream: a #GMimeStreamFile
  *
  * Gets whether or not @stream owns the backend FILE pointer.
  *
@@ -388,8 +439,8 @@ g_mime_stream_file_get_owner (GMimeStreamFile *stream)
 
 /**
  * g_mime_stream_file_set_owner:
- * @stream: file stream
- * @owner: owner
+ * @stream: a #GMimeStreamFile
+ * @owner: %TRUE if this stream should own the FILE pointer or %FALSE otherwise
  *
  * Sets whether or not @stream owns the backend FILE pointer.
  *
