@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*  GMime
- *  Copyright (C) 2000-2008 Jeffrey Stedfast
+ *  Copyright (C) 2000-2009 Jeffrey Stedfast
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Lesser General Public License
@@ -23,16 +23,29 @@
 #include <config.h>
 #endif
 
+#include <glib.h>
+
 #include <sys/types.h>
 #include <sys/stat.h>
+#ifdef HAVE_UNISTD_H
+#include <unistd.h>
+#endif
 #include <fcntl.h>
 #include <errno.h>
+#ifdef G_OS_WIN32
+#include <io.h>
+#endif
 
 #include "gmime-stream-fs.h"
 
-
 #ifndef HAVE_FSYNC
-int fsync (int fd) { return 0; }
+#ifdef G_OS_WIN32
+/* _commit() is the equivalent of fsync() on Windows, but it aborts the
+ * program if the fd is a tty, so we'll just no-op for now... */
+static int fsync (int fd) { return 0; }
+#else
+static int fsync (int fd) { return 0; }
+#endif
 #endif
 
 
@@ -138,8 +151,15 @@ stream_read (GMimeStream *stream, char *buf, size_t len)
 	GMimeStreamFs *fs = (GMimeStreamFs *) stream;
 	ssize_t nread;
 	
-	if (stream->bound_end != -1 && stream->position >= stream->bound_end)
+	if (fs->fd == -1) {
+		errno = EBADF;
 		return -1;
+	}
+	
+	if (stream->bound_end != -1 && stream->position >= stream->bound_end) {
+		errno = EINVAL;
+		return -1;
+	}
 	
 	if (stream->bound_end != -1)
 		len = MIN (stream->bound_end - stream->position, (gint64) len);
@@ -151,10 +171,11 @@ stream_read (GMimeStream *stream, char *buf, size_t len)
 		nread = read (fs->fd, buf, len);
 	} while (nread == -1 && errno == EINTR);
 	
-	if (nread > 0)
+	if (nread > 0) {
 		stream->position += nread;
-	else if (nread == 0)
+	} else if (nread == 0) {
 		fs->eos = TRUE;
+	}
 	
 	return nread;
 }
@@ -166,8 +187,15 @@ stream_write (GMimeStream *stream, const char *buf, size_t len)
 	size_t nwritten = 0;
 	ssize_t n;
 	
-	if (stream->bound_end != -1 && stream->position >= stream->bound_end)
+	if (fs->fd == -1) {
+		errno = EBADF;
 		return -1;
+	}
+	
+	if (stream->bound_end != -1 && stream->position >= stream->bound_end) {
+		errno = EINVAL;
+		return -1;
+	}
 	
 	if (stream->bound_end != -1)
 		len = MIN (stream->bound_end - stream->position, (gint64) len);
@@ -187,10 +215,12 @@ stream_write (GMimeStream *stream, const char *buf, size_t len)
 	if (n == -1 && (errno == EFBIG || errno == ENOSPC))
 		fs->eos = TRUE;
 	
-	if (nwritten > 0)
+	if (nwritten > 0) {
 		stream->position += nwritten;
-	else if (n == -1)
+	} else if (n == -1) {
+		/* error and nothing written */
 		return -1;
+	}
 	
 	return nwritten;
 }
@@ -200,7 +230,10 @@ stream_flush (GMimeStream *stream)
 {
 	GMimeStreamFs *fs = (GMimeStreamFs *) stream;
 	
-	g_return_val_if_fail (fs->fd != -1, -1);
+	if (fs->fd == -1) {
+		errno = EBADF;
+		return -1;
+	}
 	
 	return fsync (fs->fd);
 }
@@ -227,7 +260,8 @@ stream_eos (GMimeStream *stream)
 {
 	GMimeStreamFs *fs = (GMimeStreamFs *) stream;
 	
-	g_return_val_if_fail (fs->fd != -1, TRUE);
+	if (fs->fd == -1)
+		return TRUE;
 	
 	return fs->eos;
 }
@@ -237,8 +271,10 @@ stream_reset (GMimeStream *stream)
 {
 	GMimeStreamFs *fs = (GMimeStreamFs *) stream;
 	
-	if (fs->fd == -1)
+	if (fs->fd == -1) {
+		errno = EBADF;
 		return -1;
+	}
 	
 	if (stream->position == stream->bound_start) {
 		fs->eos = FALSE;
@@ -262,7 +298,10 @@ stream_seek (GMimeStream *stream, gint64 offset, GMimeSeekWhence whence)
 	GMimeStreamFs *fs = (GMimeStreamFs *) stream;
 	gint64 real;
 	
-	g_return_val_if_fail (fs->fd != -1, -1);
+	if (fs->fd == -1) {
+		errno = EBADF;
+		return -1;
+	}
 	
 	switch (whence) {
 	case GMIME_STREAM_SEEK_SET:
@@ -295,15 +334,19 @@ stream_seek (GMimeStream *stream, gint64 offset, GMimeSeekWhence whence)
 	}
 	
 	/* sanity check the resultant offset */
-	if (real < stream->bound_start)
+	if (real < stream->bound_start) {
+		errno = EINVAL;
 		return -1;
+	}
 	
 	/* short-cut if we are seeking to our current position */
 	if (real == stream->position)
 		return real;
 	
-	if (stream->bound_end != -1 && real > stream->bound_end)
+	if (stream->bound_end != -1 && real > stream->bound_end) {
+		errno = EINVAL;
 		return -1;
+	}
 	
 	if ((real = lseek (fs->fd, (off_t) real, SEEK_SET)) == -1)
 		return -1;
@@ -330,14 +373,21 @@ stream_length (GMimeStream *stream)
 	GMimeStreamFs *fs = (GMimeStreamFs *) stream;
 	gint64 bound_end;
 	
+	if (fs->fd == -1) {
+		errno = EBADF;
+		return -1;
+	}
+	
 	if (stream->bound_end != -1)
 		return stream->bound_end - stream->bound_start;
 	
 	bound_end = lseek (fs->fd, (off_t) 0, SEEK_END);
 	lseek (fs->fd, (off_t) stream->position, SEEK_SET);
 	
-	if (bound_end < stream->bound_start)
+	if (bound_end < stream->bound_start) {
+		errno = EINVAL;
 		return -1;
+	}
 	
 	return bound_end - stream->bound_start;
 }
@@ -361,7 +411,7 @@ stream_substream (GMimeStream *stream, gint64 start, gint64 end)
  * g_mime_stream_fs_new:
  * @fd: file descriptor
  *
- * Creates a new GMimeStreamFs object around @fd.
+ * Creates a new #GMimeStreamFs object around @fd.
  *
  * Returns: a stream using @fd.
  **/
@@ -371,6 +421,10 @@ g_mime_stream_fs_new (int fd)
 	GMimeStreamFs *fs;
 	gint64 start;
 	
+#ifdef G_OS_WIN32
+	_setmode (fd, O_BINARY);
+#endif
+	
 	if ((start = lseek (fd, (off_t) 0, SEEK_CUR)) == -1)
 		start = 0;
 	
@@ -386,11 +440,11 @@ g_mime_stream_fs_new (int fd)
 
 /**
  * g_mime_stream_fs_new_with_bounds:
- * @fd: file descriptor
+ * @fd: a file descriptor
  * @start: start boundary
  * @end: end boundary
  *
- * Creates a new GMimeStreamFs object around @fd with bounds @start
+ * Creates a new #GMimeStreamFs object around @fd with bounds @start
  * and @end.
  *
  * Returns: a stream using @fd with bounds @start and @end.
@@ -400,6 +454,10 @@ g_mime_stream_fs_new_with_bounds (int fd, gint64 start, gint64 end)
 {
 	GMimeStreamFs *fs;
 	
+#ifdef G_OS_WIN32
+	_setmode (fd, O_BINARY);
+#endif
+	
 	fs = g_object_newv (GMIME_TYPE_STREAM_FS, 0, NULL);
 	g_mime_stream_construct (GMIME_STREAM (fs), start, end);
 	fs->owner = TRUE;
@@ -412,7 +470,7 @@ g_mime_stream_fs_new_with_bounds (int fd, gint64 start, gint64 end)
 
 /**
  * g_mime_stream_fs_get_owner:
- * @stream: fs stream
+ * @stream: a #GMimeStreamFs
  *
  * Gets whether or not @stream owns the backend file descriptor.
  *
@@ -430,10 +488,10 @@ g_mime_stream_fs_get_owner (GMimeStreamFs *stream)
 
 /**
  * g_mime_stream_fs_set_owner:
- * @stream: fs stream
- * @owner: owner
+ * @stream: a #GMimeStreamFs
+ * @owner: %TRUE if this stream should own the file descriptor or %FALSE otherwise
  *
- * Sets whether or not @stream owns the backend FS pointer.
+ * Sets whether or not @stream owns the backend file descriptor.
  *
  * Note: @owner should be %TRUE if the stream should close() the
  * backend file descriptor when destroyed or %FALSE otherwise.
