@@ -1,27 +1,27 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*  GMime
- *  Copyright (C) 2000-2007 Jeffrey Stedfast
+ *  Copyright (C) 2000-2009 Jeffrey Stedfast
  *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
- *  (at your option) any later version.
+ *  This library is free software; you can redistribute it and/or
+ *  modify it under the terms of the GNU Lesser General Public License
+ *  as published by the Free Software Foundation; either version 2.1
+ *  of the License, or (at your option) any later version.
  *
- *  This program is distributed in the hope that it will be useful,
+ *  This library is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ *  Lesser General Public License for more details.
  *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+ *  You should have received a copy of the GNU Lesser General Public
+ *  License along with this library; if not, write to the Free
+ *  Software Foundation, 51 Franklin Street, Fifth Floor, Boston, MA
+ *  02110-1301, USA.
  */
 
 
 #ifndef __GMIME_STREAM_FS_H__
 #define __GMIME_STREAM_FS_H__
 
-#include <unistd.h>
 #include <gmime/gmime-stream.h>
 
 G_BEGIN_DECLS
@@ -36,6 +36,15 @@ G_BEGIN_DECLS
 typedef struct _GMimeStreamFs GMimeStreamFs;
 typedef struct _GMimeStreamFsClass GMimeStreamFsClass;
 
+/**
+ * GMimeStreamFs:
+ * @parent_object: parent #GMimeStream
+ * @owner: %TRUE if this stream owns @fd
+ * @eos: %TRUE if end-of-stream
+ * @fd: file descriptor
+ *
+ * A #GMimeStream wrapper around POSIX file descriptors.
+ **/
 struct _GMimeStreamFs {
 	GMimeStream parent_object;
 	
@@ -53,7 +62,7 @@ struct _GMimeStreamFsClass {
 GType g_mime_stream_fs_get_type (void);
 
 GMimeStream *g_mime_stream_fs_new (int fd);
-GMimeStream *g_mime_stream_fs_new_with_bounds (int fd, off_t start, off_t end);
+GMimeStream *g_mime_stream_fs_new_with_bounds (int fd, gint64 start, gint64 end);
 
 gboolean g_mime_stream_fs_get_owner (GMimeStreamFs *stream);
 void g_mime_stream_fs_set_owner (GMimeStreamFs *stream, gboolean owner);
